# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("tdh.dll")
prototypes = \
    {
        #
        'TdhCreatePayloadFilter': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"FieldName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "CompareOp": SimTypeShort(signed=False, label="UInt16"), "Value": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PAYLOAD_FILTER_PREDICATE", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ProviderGuid", "EventDescriptor", "EventMatchANY", "PayloadPredicateCount", "PayloadPredicates", "PayloadFilter"]),
        #
        'TdhDeletePayloadFilter': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PayloadFilter"]),
        #
        'TdhAggregatePayloadFilters': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), label="LPArray", offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimStruct({"Ptr": SimTypeLongLong(signed=False, label="UInt64"), "Size": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="UInt32")}, name="EVENT_FILTER_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PayloadFilterCount", "PayloadFilterPtrs", "EventMatchALLFlags", "EventFilterDescriptor"]),
        #
        'TdhCleanupPayloadEventFilterDescriptor': SimTypeFunction([SimTypePointer(SimStruct({"Ptr": SimTypeLongLong(signed=False, label="UInt64"), "Size": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="UInt32")}, name="EVENT_FILTER_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["EventFilterDescriptor"]),
        #
        'TdhGetEventInformation': SimTypeFunction([SimTypePointer(SimStruct({"EventHeader": SimStruct({"Size": SimTypeShort(signed=False, label="UInt16"), "HeaderType": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16"), "EventProperty": SimTypeShort(signed=False, label="UInt16"), "ThreadId": SimTypeInt(signed=False, label="UInt32"), "ProcessId": SimTypeInt(signed=False, label="UInt32"), "TimeStamp": SimTypeBottom(label="LARGE_INTEGER"), "ProviderId": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "Anonymous": SimUnion({"Anonymous": SimStruct({"KernelTime": SimTypeInt(signed=False, label="UInt32"), "UserTime": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ActivityId": SimTypeBottom(label="Guid")}, name="EVENT_HEADER", pack=False, align=None), "BufferContext": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"ProcessorNumber": SimTypeChar(label="Byte"), "Alignment": SimTypeChar(label="Byte")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "LoggerId": SimTypeShort(signed=False, label="UInt16")}, name="ETW_BUFFER_CONTEXT", pack=False, align=None), "ExtendedDataCount": SimTypeShort(signed=False, label="UInt16"), "UserDataLength": SimTypeShort(signed=False, label="UInt16"), "ExtendedData": SimTypePointer(SimStruct({"Reserved1": SimTypeShort(signed=False, label="UInt16"), "ExtType": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct({"_bitfield": SimTypeShort(signed=False, label="UInt16")}, name="_Anonymous_e__Struct", pack=False, align=None), "DataSize": SimTypeShort(signed=False, label="UInt16"), "DataPtr": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_HEADER_EXTENDED_DATA_ITEM", pack=False, align=None), offset=0), "UserData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="EVENT_RECORD", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"ParameterValue": SimTypeLongLong(signed=False, label="UInt64"), "ParameterType": SimTypeInt(signed=False, label="TDH_CONTEXT_TYPE"), "ParameterSize": SimTypeInt(signed=False, label="UInt32")}, name="TDH_CONTEXT", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimStruct({"ProviderGuid": SimTypeBottom(label="Guid"), "EventGuid": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "DecodingSource": SimTypeInt(signed=False, label="DECODING_SOURCE"), "ProviderNameOffset": SimTypeInt(signed=False, label="UInt32"), "LevelNameOffset": SimTypeInt(signed=False, label="UInt32"), "ChannelNameOffset": SimTypeInt(signed=False, label="UInt32"), "KeywordsNameOffset": SimTypeInt(signed=False, label="UInt32"), "TaskNameOffset": SimTypeInt(signed=False, label="UInt32"), "OpcodeNameOffset": SimTypeInt(signed=False, label="UInt32"), "EventMessageOffset": SimTypeInt(signed=False, label="UInt32"), "ProviderMessageOffset": SimTypeInt(signed=False, label="UInt32"), "BinaryXMLOffset": SimTypeInt(signed=False, label="UInt32"), "BinaryXMLSize": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimUnion({"EventNameOffset": SimTypeInt(signed=False, label="UInt32"), "ActivityIDNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "Anonymous2": SimUnion({"EventAttributesOffset": SimTypeInt(signed=False, label="UInt32"), "RelatedActivityIDNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "PropertyCount": SimTypeInt(signed=False, label="UInt32"), "TopLevelPropertyCount": SimTypeInt(signed=False, label="UInt32"), "Anonymous3": SimUnion({"Flags": SimTypeInt(signed=False, label="TEMPLATE_FLAGS"), "Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "EventPropertyInfoArray": SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="PROPERTY_FLAGS"), "NameOffset": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimUnion({"nonStructType": SimStruct({"InType": SimTypeShort(signed=False, label="UInt16"), "OutType": SimTypeShort(signed=False, label="UInt16"), "MapNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="_nonStructType", pack=False, align=None), "structType": SimStruct({"StructStartIndex": SimTypeShort(signed=False, label="UInt16"), "NumOfStructMembers": SimTypeShort(signed=False, label="UInt16"), "padding": SimTypeInt(signed=False, label="UInt32")}, name="_structType", pack=False, align=None), "customSchemaType": SimStruct({"InType": SimTypeShort(signed=False, label="UInt16"), "OutType": SimTypeShort(signed=False, label="UInt16"), "CustomSchemaOffset": SimTypeInt(signed=False, label="UInt32")}, name="_customSchemaType", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"count": SimTypeShort(signed=False, label="UInt16"), "countPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "Anonymous3": SimUnion({"length": SimTypeShort(signed=False, label="UInt16"), "lengthPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "Anonymous4": SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="EVENT_PROPERTY_INFO", pack=False, align=None), offset=0)}, name="TRACE_EVENT_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Event", "TdhContextCount", "TdhContext", "Buffer", "BufferSize"]),
        #
        'TdhGetEventMapInformation': SimTypeFunction([SimTypePointer(SimStruct({"EventHeader": SimStruct({"Size": SimTypeShort(signed=False, label="UInt16"), "HeaderType": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16"), "EventProperty": SimTypeShort(signed=False, label="UInt16"), "ThreadId": SimTypeInt(signed=False, label="UInt32"), "ProcessId": SimTypeInt(signed=False, label="UInt32"), "TimeStamp": SimTypeBottom(label="LARGE_INTEGER"), "ProviderId": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "Anonymous": SimUnion({"Anonymous": SimStruct({"KernelTime": SimTypeInt(signed=False, label="UInt32"), "UserTime": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ActivityId": SimTypeBottom(label="Guid")}, name="EVENT_HEADER", pack=False, align=None), "BufferContext": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"ProcessorNumber": SimTypeChar(label="Byte"), "Alignment": SimTypeChar(label="Byte")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "LoggerId": SimTypeShort(signed=False, label="UInt16")}, name="ETW_BUFFER_CONTEXT", pack=False, align=None), "ExtendedDataCount": SimTypeShort(signed=False, label="UInt16"), "UserDataLength": SimTypeShort(signed=False, label="UInt16"), "ExtendedData": SimTypePointer(SimStruct({"Reserved1": SimTypeShort(signed=False, label="UInt16"), "ExtType": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct({"_bitfield": SimTypeShort(signed=False, label="UInt16")}, name="_Anonymous_e__Struct", pack=False, align=None), "DataSize": SimTypeShort(signed=False, label="UInt16"), "DataPtr": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_HEADER_EXTENDED_DATA_ITEM", pack=False, align=None), offset=0), "UserData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="EVENT_RECORD", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"NameOffset": SimTypeInt(signed=False, label="UInt32"), "Flag": SimTypeInt(signed=False, label="MAP_FLAGS"), "EntryCount": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"MapEntryValueType": SimTypeInt(signed=False, label="MAP_VALUETYPE"), "FormatStringOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "MapEntryArray": SimTypePointer(SimStruct({"OutputOffset": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"Value": SimTypeInt(signed=False, label="UInt32"), "InputOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="EVENT_MAP_ENTRY", pack=False, align=None), offset=0)}, name="EVENT_MAP_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pEvent", "pMapName", "pBuffer", "pBufferSize"]),
        #
        'TdhGetPropertySize': SimTypeFunction([SimTypePointer(SimStruct({"EventHeader": SimStruct({"Size": SimTypeShort(signed=False, label="UInt16"), "HeaderType": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16"), "EventProperty": SimTypeShort(signed=False, label="UInt16"), "ThreadId": SimTypeInt(signed=False, label="UInt32"), "ProcessId": SimTypeInt(signed=False, label="UInt32"), "TimeStamp": SimTypeBottom(label="LARGE_INTEGER"), "ProviderId": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "Anonymous": SimUnion({"Anonymous": SimStruct({"KernelTime": SimTypeInt(signed=False, label="UInt32"), "UserTime": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ActivityId": SimTypeBottom(label="Guid")}, name="EVENT_HEADER", pack=False, align=None), "BufferContext": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"ProcessorNumber": SimTypeChar(label="Byte"), "Alignment": SimTypeChar(label="Byte")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "LoggerId": SimTypeShort(signed=False, label="UInt16")}, name="ETW_BUFFER_CONTEXT", pack=False, align=None), "ExtendedDataCount": SimTypeShort(signed=False, label="UInt16"), "UserDataLength": SimTypeShort(signed=False, label="UInt16"), "ExtendedData": SimTypePointer(SimStruct({"Reserved1": SimTypeShort(signed=False, label="UInt16"), "ExtType": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct({"_bitfield": SimTypeShort(signed=False, label="UInt16")}, name="_Anonymous_e__Struct", pack=False, align=None), "DataSize": SimTypeShort(signed=False, label="UInt16"), "DataPtr": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_HEADER_EXTENDED_DATA_ITEM", pack=False, align=None), offset=0), "UserData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="EVENT_RECORD", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"ParameterValue": SimTypeLongLong(signed=False, label="UInt64"), "ParameterType": SimTypeInt(signed=False, label="TDH_CONTEXT_TYPE"), "ParameterSize": SimTypeInt(signed=False, label="UInt32")}, name="TDH_CONTEXT", pack=False, align=None), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"PropertyName": SimTypeLongLong(signed=False, label="UInt64"), "ArrayIndex": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeInt(signed=False, label="UInt32")}, name="PROPERTY_DATA_DESCRIPTOR", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pEvent", "TdhContextCount", "pTdhContext", "PropertyDataCount", "pPropertyData", "pPropertySize"]),
        #
        'TdhGetProperty': SimTypeFunction([SimTypePointer(SimStruct({"EventHeader": SimStruct({"Size": SimTypeShort(signed=False, label="UInt16"), "HeaderType": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16"), "EventProperty": SimTypeShort(signed=False, label="UInt16"), "ThreadId": SimTypeInt(signed=False, label="UInt32"), "ProcessId": SimTypeInt(signed=False, label="UInt32"), "TimeStamp": SimTypeBottom(label="LARGE_INTEGER"), "ProviderId": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "Anonymous": SimUnion({"Anonymous": SimStruct({"KernelTime": SimTypeInt(signed=False, label="UInt32"), "UserTime": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ActivityId": SimTypeBottom(label="Guid")}, name="EVENT_HEADER", pack=False, align=None), "BufferContext": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"ProcessorNumber": SimTypeChar(label="Byte"), "Alignment": SimTypeChar(label="Byte")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "LoggerId": SimTypeShort(signed=False, label="UInt16")}, name="ETW_BUFFER_CONTEXT", pack=False, align=None), "ExtendedDataCount": SimTypeShort(signed=False, label="UInt16"), "UserDataLength": SimTypeShort(signed=False, label="UInt16"), "ExtendedData": SimTypePointer(SimStruct({"Reserved1": SimTypeShort(signed=False, label="UInt16"), "ExtType": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct({"_bitfield": SimTypeShort(signed=False, label="UInt16")}, name="_Anonymous_e__Struct", pack=False, align=None), "DataSize": SimTypeShort(signed=False, label="UInt16"), "DataPtr": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_HEADER_EXTENDED_DATA_ITEM", pack=False, align=None), offset=0), "UserData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="EVENT_RECORD", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"ParameterValue": SimTypeLongLong(signed=False, label="UInt64"), "ParameterType": SimTypeInt(signed=False, label="TDH_CONTEXT_TYPE"), "ParameterSize": SimTypeInt(signed=False, label="UInt32")}, name="TDH_CONTEXT", pack=False, align=None), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"PropertyName": SimTypeLongLong(signed=False, label="UInt64"), "ArrayIndex": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeInt(signed=False, label="UInt32")}, name="PROPERTY_DATA_DESCRIPTOR", pack=False, align=None), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pEvent", "TdhContextCount", "pTdhContext", "PropertyDataCount", "pPropertyData", "BufferSize", "pBuffer"]),
        #
        'TdhEnumerateProviders': SimTypeFunction([SimTypePointer(SimStruct({"NumberOfProviders": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeInt(signed=False, label="UInt32"), "TraceProviderInfoArray": SimTypePointer(SimStruct({"ProviderGuid": SimTypeBottom(label="Guid"), "SchemaSource": SimTypeInt(signed=False, label="UInt32"), "ProviderNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="TRACE_PROVIDER_INFO", pack=False, align=None), offset=0)}, name="PROVIDER_ENUMERATION_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pBuffer", "pBufferSize"]),
        #
        'TdhQueryProviderFieldInformation': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="EVENT_FIELD_TYPE"), SimTypePointer(SimStruct({"NumberOfElements": SimTypeInt(signed=False, label="UInt32"), "FieldType": SimTypeInt(signed=False, label="EVENT_FIELD_TYPE"), "FieldInfoArray": SimTypePointer(SimStruct({"NameOffset": SimTypeInt(signed=False, label="UInt32"), "DescriptionOffset": SimTypeInt(signed=False, label="UInt32"), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="PROVIDER_FIELD_INFO", pack=False, align=None), offset=0)}, name="PROVIDER_FIELD_INFOARRAY", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pGuid", "EventFieldValue", "EventFieldType", "pBuffer", "pBufferSize"]),
        #
        'TdhEnumerateProviderFieldInformation': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="EVENT_FIELD_TYPE"), SimTypePointer(SimStruct({"NumberOfElements": SimTypeInt(signed=False, label="UInt32"), "FieldType": SimTypeInt(signed=False, label="EVENT_FIELD_TYPE"), "FieldInfoArray": SimTypePointer(SimStruct({"NameOffset": SimTypeInt(signed=False, label="UInt32"), "DescriptionOffset": SimTypeInt(signed=False, label="UInt32"), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="PROVIDER_FIELD_INFO", pack=False, align=None), offset=0)}, name="PROVIDER_FIELD_INFOARRAY", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pGuid", "EventFieldType", "pBuffer", "pBufferSize"]),
        #
        'TdhEnumerateProviderFilters': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"ParameterValue": SimTypeLongLong(signed=False, label="UInt64"), "ParameterType": SimTypeInt(signed=False, label="TDH_CONTEXT_TYPE"), "ParameterSize": SimTypeInt(signed=False, label="UInt32")}, name="TDH_CONTEXT", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Id": SimTypeChar(label="Byte"), "Version": SimTypeChar(label="Byte"), "MessageOffset": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeInt(signed=False, label="UInt32"), "PropertyCount": SimTypeInt(signed=False, label="UInt32"), "EventPropertyInfoArray": SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="PROPERTY_FLAGS"), "NameOffset": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimUnion({"nonStructType": SimStruct({"InType": SimTypeShort(signed=False, label="UInt16"), "OutType": SimTypeShort(signed=False, label="UInt16"), "MapNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="_nonStructType", pack=False, align=None), "structType": SimStruct({"StructStartIndex": SimTypeShort(signed=False, label="UInt16"), "NumOfStructMembers": SimTypeShort(signed=False, label="UInt16"), "padding": SimTypeInt(signed=False, label="UInt32")}, name="_structType", pack=False, align=None), "customSchemaType": SimStruct({"InType": SimTypeShort(signed=False, label="UInt16"), "OutType": SimTypeShort(signed=False, label="UInt16"), "CustomSchemaOffset": SimTypeInt(signed=False, label="UInt32")}, name="_customSchemaType", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"count": SimTypeShort(signed=False, label="UInt16"), "countPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "Anonymous3": SimUnion({"length": SimTypeShort(signed=False, label="UInt16"), "lengthPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "Anonymous4": SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="EVENT_PROPERTY_INFO", pack=False, align=None), offset=0)}, name="PROVIDER_FILTER_INFO", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Guid", "TdhContextCount", "TdhContext", "FilterCount", "Buffer", "BufferSize"]),
        #
        'TdhLoadManifest': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Manifest"]),
        #
        'TdhLoadManifestFromMemory': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pData", "cbData"]),
        #
        'TdhUnloadManifest': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Manifest"]),
        #
        'TdhUnloadManifestFromMemory': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pData", "cbData"]),
        #
        'TdhFormatProperty': SimTypeFunction([SimTypePointer(SimStruct({"ProviderGuid": SimTypeBottom(label="Guid"), "EventGuid": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "DecodingSource": SimTypeInt(signed=False, label="DECODING_SOURCE"), "ProviderNameOffset": SimTypeInt(signed=False, label="UInt32"), "LevelNameOffset": SimTypeInt(signed=False, label="UInt32"), "ChannelNameOffset": SimTypeInt(signed=False, label="UInt32"), "KeywordsNameOffset": SimTypeInt(signed=False, label="UInt32"), "TaskNameOffset": SimTypeInt(signed=False, label="UInt32"), "OpcodeNameOffset": SimTypeInt(signed=False, label="UInt32"), "EventMessageOffset": SimTypeInt(signed=False, label="UInt32"), "ProviderMessageOffset": SimTypeInt(signed=False, label="UInt32"), "BinaryXMLOffset": SimTypeInt(signed=False, label="UInt32"), "BinaryXMLSize": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimUnion({"EventNameOffset": SimTypeInt(signed=False, label="UInt32"), "ActivityIDNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "Anonymous2": SimUnion({"EventAttributesOffset": SimTypeInt(signed=False, label="UInt32"), "RelatedActivityIDNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "PropertyCount": SimTypeInt(signed=False, label="UInt32"), "TopLevelPropertyCount": SimTypeInt(signed=False, label="UInt32"), "Anonymous3": SimUnion({"Flags": SimTypeInt(signed=False, label="TEMPLATE_FLAGS"), "Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "EventPropertyInfoArray": SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="PROPERTY_FLAGS"), "NameOffset": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimUnion({"nonStructType": SimStruct({"InType": SimTypeShort(signed=False, label="UInt16"), "OutType": SimTypeShort(signed=False, label="UInt16"), "MapNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="_nonStructType", pack=False, align=None), "structType": SimStruct({"StructStartIndex": SimTypeShort(signed=False, label="UInt16"), "NumOfStructMembers": SimTypeShort(signed=False, label="UInt16"), "padding": SimTypeInt(signed=False, label="UInt32")}, name="_structType", pack=False, align=None), "customSchemaType": SimStruct({"InType": SimTypeShort(signed=False, label="UInt16"), "OutType": SimTypeShort(signed=False, label="UInt16"), "CustomSchemaOffset": SimTypeInt(signed=False, label="UInt32")}, name="_customSchemaType", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"count": SimTypeShort(signed=False, label="UInt16"), "countPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "Anonymous3": SimUnion({"length": SimTypeShort(signed=False, label="UInt16"), "lengthPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "Anonymous4": SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="EVENT_PROPERTY_INFO", pack=False, align=None), offset=0)}, name="TRACE_EVENT_INFO", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"NameOffset": SimTypeInt(signed=False, label="UInt32"), "Flag": SimTypeInt(signed=False, label="MAP_FLAGS"), "EntryCount": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"MapEntryValueType": SimTypeInt(signed=False, label="MAP_VALUETYPE"), "FormatStringOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "MapEntryArray": SimTypePointer(SimStruct({"OutputOffset": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"Value": SimTypeInt(signed=False, label="UInt32"), "InputOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="EVENT_MAP_ENTRY", pack=False, align=None), offset=0)}, name="EVENT_MAP_INFO", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["EventInfo", "MapInfo", "PointerSize", "PropertyInType", "PropertyOutType", "PropertyLength", "UserDataLength", "UserData", "BufferSize", "Buffer", "UserDataConsumed"]),
        #
        'TdhOpenDecodingHandle': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Handle"]),
        #
        'TdhSetDecodingParameter': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"ParameterValue": SimTypeLongLong(signed=False, label="UInt64"), "ParameterType": SimTypeInt(signed=False, label="TDH_CONTEXT_TYPE"), "ParameterSize": SimTypeInt(signed=False, label="UInt32")}, name="TDH_CONTEXT", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Handle", "TdhContext"]),
        #
        'TdhGetDecodingParameter': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"ParameterValue": SimTypeLongLong(signed=False, label="UInt64"), "ParameterType": SimTypeInt(signed=False, label="TDH_CONTEXT_TYPE"), "ParameterSize": SimTypeInt(signed=False, label="UInt32")}, name="TDH_CONTEXT", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Handle", "TdhContext"]),
        #
        'TdhGetWppProperty': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"EventHeader": SimStruct({"Size": SimTypeShort(signed=False, label="UInt16"), "HeaderType": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16"), "EventProperty": SimTypeShort(signed=False, label="UInt16"), "ThreadId": SimTypeInt(signed=False, label="UInt32"), "ProcessId": SimTypeInt(signed=False, label="UInt32"), "TimeStamp": SimTypeBottom(label="LARGE_INTEGER"), "ProviderId": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "Anonymous": SimUnion({"Anonymous": SimStruct({"KernelTime": SimTypeInt(signed=False, label="UInt32"), "UserTime": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ActivityId": SimTypeBottom(label="Guid")}, name="EVENT_HEADER", pack=False, align=None), "BufferContext": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"ProcessorNumber": SimTypeChar(label="Byte"), "Alignment": SimTypeChar(label="Byte")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "LoggerId": SimTypeShort(signed=False, label="UInt16")}, name="ETW_BUFFER_CONTEXT", pack=False, align=None), "ExtendedDataCount": SimTypeShort(signed=False, label="UInt16"), "UserDataLength": SimTypeShort(signed=False, label="UInt16"), "ExtendedData": SimTypePointer(SimStruct({"Reserved1": SimTypeShort(signed=False, label="UInt16"), "ExtType": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct({"_bitfield": SimTypeShort(signed=False, label="UInt16")}, name="_Anonymous_e__Struct", pack=False, align=None), "DataSize": SimTypeShort(signed=False, label="UInt16"), "DataPtr": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_HEADER_EXTENDED_DATA_ITEM", pack=False, align=None), offset=0), "UserData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="EVENT_RECORD", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Handle", "EventRecord", "PropertyName", "BufferSize", "Buffer"]),
        #
        'TdhGetWppMessage': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"EventHeader": SimStruct({"Size": SimTypeShort(signed=False, label="UInt16"), "HeaderType": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16"), "EventProperty": SimTypeShort(signed=False, label="UInt16"), "ThreadId": SimTypeInt(signed=False, label="UInt32"), "ProcessId": SimTypeInt(signed=False, label="UInt32"), "TimeStamp": SimTypeBottom(label="LARGE_INTEGER"), "ProviderId": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "Anonymous": SimUnion({"Anonymous": SimStruct({"KernelTime": SimTypeInt(signed=False, label="UInt32"), "UserTime": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ActivityId": SimTypeBottom(label="Guid")}, name="EVENT_HEADER", pack=False, align=None), "BufferContext": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"ProcessorNumber": SimTypeChar(label="Byte"), "Alignment": SimTypeChar(label="Byte")}, name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "LoggerId": SimTypeShort(signed=False, label="UInt16")}, name="ETW_BUFFER_CONTEXT", pack=False, align=None), "ExtendedDataCount": SimTypeShort(signed=False, label="UInt16"), "UserDataLength": SimTypeShort(signed=False, label="UInt16"), "ExtendedData": SimTypePointer(SimStruct({"Reserved1": SimTypeShort(signed=False, label="UInt16"), "ExtType": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct({"_bitfield": SimTypeShort(signed=False, label="UInt16")}, name="_Anonymous_e__Struct", pack=False, align=None), "DataSize": SimTypeShort(signed=False, label="UInt16"), "DataPtr": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_HEADER_EXTENDED_DATA_ITEM", pack=False, align=None), offset=0), "UserData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="EVENT_RECORD", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Handle", "EventRecord", "BufferSize", "Buffer"]),
        #
        'TdhCloseDecodingHandle': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Handle"]),
        #
        'TdhLoadManifestFromBinary': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["BinaryPath"]),
        #
        'TdhEnumerateManifestProviderEvents': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"NumberOfEvents": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeInt(signed=False, label="UInt32"), "EventDescriptorsArray": SimTypePointer(SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), offset=0)}, name="PROVIDER_EVENT_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ProviderGuid", "Buffer", "BufferSize"]),
        #
        'TdhGetManifestEventInformation': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"ProviderGuid": SimTypeBottom(label="Guid"), "EventGuid": SimTypeBottom(label="Guid"), "EventDescriptor": SimStruct({"Id": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeChar(label="Byte"), "Channel": SimTypeChar(label="Byte"), "Level": SimTypeChar(label="Byte"), "Opcode": SimTypeChar(label="Byte"), "Task": SimTypeShort(signed=False, label="UInt16"), "Keyword": SimTypeLongLong(signed=False, label="UInt64")}, name="EVENT_DESCRIPTOR", pack=False, align=None), "DecodingSource": SimTypeInt(signed=False, label="DECODING_SOURCE"), "ProviderNameOffset": SimTypeInt(signed=False, label="UInt32"), "LevelNameOffset": SimTypeInt(signed=False, label="UInt32"), "ChannelNameOffset": SimTypeInt(signed=False, label="UInt32"), "KeywordsNameOffset": SimTypeInt(signed=False, label="UInt32"), "TaskNameOffset": SimTypeInt(signed=False, label="UInt32"), "OpcodeNameOffset": SimTypeInt(signed=False, label="UInt32"), "EventMessageOffset": SimTypeInt(signed=False, label="UInt32"), "ProviderMessageOffset": SimTypeInt(signed=False, label="UInt32"), "BinaryXMLOffset": SimTypeInt(signed=False, label="UInt32"), "BinaryXMLSize": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimUnion({"EventNameOffset": SimTypeInt(signed=False, label="UInt32"), "ActivityIDNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "Anonymous2": SimUnion({"EventAttributesOffset": SimTypeInt(signed=False, label="UInt32"), "RelatedActivityIDNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "PropertyCount": SimTypeInt(signed=False, label="UInt32"), "TopLevelPropertyCount": SimTypeInt(signed=False, label="UInt32"), "Anonymous3": SimUnion({"Flags": SimTypeInt(signed=False, label="TEMPLATE_FLAGS"), "Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "EventPropertyInfoArray": SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="PROPERTY_FLAGS"), "NameOffset": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimUnion({"nonStructType": SimStruct({"InType": SimTypeShort(signed=False, label="UInt16"), "OutType": SimTypeShort(signed=False, label="UInt16"), "MapNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="_nonStructType", pack=False, align=None), "structType": SimStruct({"StructStartIndex": SimTypeShort(signed=False, label="UInt16"), "NumOfStructMembers": SimTypeShort(signed=False, label="UInt16"), "padding": SimTypeInt(signed=False, label="UInt32")}, name="_structType", pack=False, align=None), "customSchemaType": SimStruct({"InType": SimTypeShort(signed=False, label="UInt16"), "OutType": SimTypeShort(signed=False, label="UInt16"), "CustomSchemaOffset": SimTypeInt(signed=False, label="UInt32")}, name="_customSchemaType", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"count": SimTypeShort(signed=False, label="UInt16"), "countPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "Anonymous3": SimUnion({"length": SimTypeShort(signed=False, label="UInt16"), "lengthPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"), "Anonymous4": SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="EVENT_PROPERTY_INFO", pack=False, align=None), offset=0)}, name="TRACE_EVENT_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ProviderGuid", "EventDescriptor", "Buffer", "BufferSize"]),
    }

lib.set_prototypes(prototypes)



# pylint:disable=unnecessary-pass
import logging
from typing import Optional, Any, TYPE_CHECKING

from ailment.statement import ConditionalJump, Assignment, Statement
from ailment.expression import Const, ITE, Expression

from ....analyses import ReachingDefinitionsAnalysis
from ....code_location import CodeLocation
from ..region_walker import RegionWalker
from ..ail_simplifier import AILBlockWalker
from ..condition_processor import ConditionProcessor
from ..structuring.structurer_nodes import EmptyBlockNotice
from .optimization_pass import OptimizationPass, OptimizationPassStage

if TYPE_CHECKING:
    from ailment import Block as AILBlock


_l = logging.getLogger(__name__)


class NodeFoundNotification(Exception):
    """
    A notification that the target node has been found.
    """

    pass


class BlockLocator(RegionWalker):
    """
    Recursively locate block in a GraphRegion instance.

    It might be reasonable to move this class into its own file.
    """

    def __init__(self, block):
        super().__init__()

        self._block = block
        self.region = None

    def walk_node(self, region, node):
        if node == self._block:
            self.region = region
            raise NodeFoundNotification()


class ExpressionReplacer(AILBlockWalker):
    """
    Replace expressions.
    """

    def __init__(self, block_addr, target_expr, callback):
        super().__init__()
        self._block_addr = block_addr
        self._target_expr = target_expr
        self._callback = callback

    def _handle_expr(
        self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement], block: Optional["AILBlock"]
    ) -> Any:
        if expr == self._target_expr:
            new_expr = self._callback(self._block_addr, stmt_idx, stmt.ins_addr, expr)
            return new_expr
        return super()._handle_expr(expr_idx, expr, stmt_idx, stmt, block)


class ITEExprConverter(OptimizationPass):
    """
    Transform specific expressions into If-Then-Else expressions, or tertiary expressions in C when
    given a single-use expression address. Requires outside analysis to provide the target expressions.
    """

    ARCHES = ["X86", "AMD64", "ARMEL", "ARMHF", "ARMCortexM", "MIPS32", "MIPS64"]
    PLATFORMS = ["windows", "linux", "cgc"]
    STAGE = OptimizationPassStage.DURING_REGION_IDENTIFICATION
    NAME = (
        "Transform single-use expressions that were assigned to in different "
        "If-Else branches into ternary expressions"
    )
    DESCRIPTION = __doc__.strip()

    def __init__(self, func, ite_exprs=None, **kwargs):
        super().__init__(func, **kwargs)
        self._ite_exprs = ite_exprs

        if self._ite_exprs:
            self.analyze()

    def _check(self):
        return True, None

    def _analyze(self, cache=None):
        for ins_addr, expr in self._ite_exprs:
            for block_addr, blocks in self.blocks_by_addr.items():
                # TODO: Optimize this stupid loop
                for block in blocks:
                    if block.addr <= ins_addr < block.addr + block.original_size:
                        block_walker = ExpressionReplacer(block_addr, expr, self._convert_expr)
                        block_walker.walk(block)

    def _convert_expr(self, block_addr: int, stmt_idx: int, ins_addr: int, atom: Expression) -> Optional[Expression]:
        rda = self.project.analyses[ReachingDefinitionsAnalysis].prep()(subject=self._func, func_graph=self._graph)

        # find the corresponding definition
        defs = []
        loc = CodeLocation(block_addr, stmt_idx, ins_addr=ins_addr)
        for def_, expr in rda.all_uses.get_uses_by_location(loc, exprs=True):
            if expr == atom:
                defs.append(def_)

        if len(defs) != 2:
            return None

        # go through all blocks in the graph to find the corresponding blocks
        def_block_addrs = {defs[0].codeloc.block_addr, defs[1].codeloc.block_addr}
        blocks = []
        for node in self._graph.nodes():
            if node.addr in def_block_addrs:
                blocks.append(node)

        if len(blocks) != 2:
            return None

        #
        # are these definitions guarded by conflicting conditions? i.e., guarded by an ite?
        #

        # find their regions
        block_0 = [b for b in blocks if b.addr == defs[0].codeloc.block_addr][0]
        region_0 = self._locate_block(block_0)

        block_1 = [b for b in blocks if b.addr == defs[1].codeloc.block_addr][0]
        region_1 = self._locate_block(block_1)

        if region_0 is None or region_1 is None or region_0 != region_1:
            return None

        # we only support the typical case of `if (cond) { a = expr_0} else { a = expr_1 }`
        head = None
        for node in region_0.graph.nodes:
            if region_0.graph.in_degree[node] == 0:
                head = node
                break
        if head is None:
            return None

        if not (region_0.graph.has_edge(head, block_0) and region_0.graph.has_edge(head, block_1)):
            return None

        #
        # create the new ITE expression to replace the atom
        #

        # extract the condition from the head
        try:
            last_stmt = ConditionProcessor.get_last_statement(head)
        except EmptyBlockNotice:
            return None
        if last_stmt is None:
            return None
        if not isinstance(last_stmt, ConditionalJump):
            return None
        if not (isinstance(last_stmt.true_target, Const) and isinstance(last_stmt.false_target, Const)):
            return None

        cond = last_stmt.condition
        if last_stmt.true_target.value == block_0.addr and last_stmt.false_target.value == block_1.addr:
            pass
        elif last_stmt.true_target.value == block_1.addr and last_stmt.false_target.value == block_0.addr:
            # swap the blocks and defs
            block_0, block_1 = block_1, block_0
            defs = [defs[1], defs[0]]
        else:
            return None

        # extract expressions from both blocks
        for idx, stmt in enumerate(reversed(block_0.statements)):
            if isinstance(stmt, Assignment) and stmt.dst.likes(atom):
                expr_0 = stmt.src
                block_0_stmt_idx = len(block_0.statements) - idx - 1
                break
        else:
            return None

        for idx, stmt in enumerate(reversed(block_1.statements)):
            if isinstance(stmt, Assignment) and stmt.dst.likes(atom):
                expr_1 = stmt.src
                block_1_stmt_idx = len(block_1.statements) - idx - 1
                break
        else:
            return None

        # make sure the two assigned dst expressions are only used once
        uses_0 = rda.all_uses.get_uses(defs[0])
        if len(uses_0) != 1:
            return None
        uses_1 = rda.all_uses.get_uses(defs[1])
        if len(uses_1) != 1:
            return None

        new_expr = ITE(
            None,
            cond,
            expr_1,
            expr_0,
            ins_addr=expr_0.ins_addr,
            vex_block_addr=expr_0.vex_block_addr,
            vex_stmt_idx=expr_0.vex_stmt_idx,
        )

        # remove the two assignments
        block_0.statements = block_0.statements[0:block_0_stmt_idx] + block_0.statements[block_0_stmt_idx + 1 :]
        block_1.statements = block_1.statements[0:block_1_stmt_idx] + block_1.statements[block_1_stmt_idx + 1 :]

        return new_expr

    def _locate_block(self, block: "AILBlock"):
        locator = BlockLocator(block)
        try:
            locator.walk(self._ri.region)
        except NodeFoundNotification:
            return locator.region
        return None  # not found



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("advpack.dll")
prototypes = \
    {
        #
        'RunSetupCommandA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "szCmdName", "szInfSection", "szDir", "lpszTitle", "phEXE", "dwFlags", "pvReserved"]),
        #
        'RunSetupCommandW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "szCmdName", "szInfSection", "szDir", "lpszTitle", "phEXE", "dwFlags", "pvReserved"]),
        #
        'NeedRebootInit': SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")),
        #
        'NeedReboot': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["dwRebootCheck"]),
        #
        'RebootCheckOnInstallA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "pszINF", "pszSec", "dwReserved"]),
        #
        'RebootCheckOnInstallW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "pszINF", "pszSec", "dwReserved"]),
        #
        'TranslateInfStringA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszInfFilename", "pszInstallSection", "pszTranslateSection", "pszTranslateKey", "pszBuffer", "cchBuffer", "pdwRequiredSize", "pvReserved"]),
        #
        'TranslateInfStringW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszInfFilename", "pszInstallSection", "pszTranslateSection", "pszTranslateKey", "pszBuffer", "cchBuffer", "pdwRequiredSize", "pvReserved"]),
        #
        'RegInstallA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"cEntries": SimTypeInt(signed=False, label="UInt32"), "pse": SimTypePointer(SimStruct({"pszName": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pszValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="STRENTRYA", pack=False, align=None), offset=0)}, name="STRTABLEA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hmod", "pszSection", "pstTable"]),
        #
        'RegInstallW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"cEntries": SimTypeInt(signed=False, label="UInt32"), "pse": SimTypePointer(SimStruct({"pszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszValue": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="STRENTRYW", pack=False, align=None), offset=0)}, name="STRTABLEW", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hmod", "pszSection", "pstTable"]),
        #
        'LaunchINFSectionExW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "hInstance", "pszParms", "nShow"]),
        #
        'ExecuteCabA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"pszCab": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pszInf": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pszSection": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "szSrcPath": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 260), "dwFlags": SimTypeInt(signed=False, label="UInt32")}, name="CABINFOA", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "pCab", "pReserved"]),
        #
        'ExecuteCabW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"pszCab": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszInf": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszSection": SimTypePointer(SimTypeChar(label="Char"), offset=0), "szSrcPath": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "dwFlags": SimTypeInt(signed=False, label="UInt32")}, name="CABINFOW", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "pCab", "pReserved"]),
        #
        'AdvInstallFileA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "lpszSourceDir", "lpszSourceFile", "lpszDestDir", "lpszDestFile", "dwFlags", "dwReserved"]),
        #
        'AdvInstallFileW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "lpszSourceDir", "lpszSourceFile", "lpszDestDir", "lpszDestFile", "dwFlags", "dwReserved"]),
        #
        'RegSaveRestoreA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pszTitleString", "hkBckupKey", "pcszRootKey", "pcszSubKey", "pcszValueName", "dwFlags"]),
        #
        'RegSaveRestoreW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pszTitleString", "hkBckupKey", "pcszRootKey", "pcszSubKey", "pcszValueName", "dwFlags"]),
        #
        'RegSaveRestoreOnINFA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pszTitle", "pszINF", "pszSection", "hHKLMBackKey", "hHKCUBackKey", "dwFlags"]),
        #
        'RegSaveRestoreOnINFW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pszTitle", "pszINF", "pszSection", "hHKLMBackKey", "hHKCUBackKey", "dwFlags"]),
        #
        'RegRestoreAllA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pszTitleString", "hkBckupKey"]),
        #
        'RegRestoreAllW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pszTitleString", "hkBckupKey"]),
        #
        'FileSaveRestoreW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hDlg", "lpFileList", "lpDir", "lpBaseName", "dwFlags"]),
        #
        'FileSaveRestoreOnINFA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pszTitle", "pszINF", "pszSection", "pszBackupDir", "pszBaseBackupFile", "dwFlags"]),
        #
        'FileSaveRestoreOnINFW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pszTitle", "pszINF", "pszSection", "pszBackupDir", "pszBaseBackupFile", "dwFlags"]),
        #
        'AddDelBackupEntryA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpcszFileList", "lpcszBackupDir", "lpcszBaseName", "dwFlags"]),
        #
        'AddDelBackupEntryW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpcszFileList", "lpcszBackupDir", "lpcszBaseName", "dwFlags"]),
        #
        'FileSaveMarkNotExistA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpFileList", "lpDir", "lpBaseName"]),
        #
        'FileSaveMarkNotExistW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpFileList", "lpDir", "lpBaseName"]),
        #
        'GetVersionFromFileA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszFilename", "pdwMSVer", "pdwLSVer", "bVersion"]),
        #
        'GetVersionFromFileW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszFilename", "pdwMSVer", "pdwLSVer", "bVersion"]),
        #
        'GetVersionFromFileExA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszFilename", "pdwMSVer", "pdwLSVer", "bVersion"]),
        #
        'GetVersionFromFileExW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszFilename", "pdwMSVer", "pdwLSVer", "bVersion"]),
        #
        'IsNTAdmin': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwReserved", "lpdwReserved"]),
        #
        'DelNodeA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pszFileOrDirName", "dwFlags"]),
        #
        'DelNodeW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pszFileOrDirName", "dwFlags"]),
        #
        'DelNodeRunDLL32W': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "hInstance", "pszParms", "nShow"]),
        #
        'OpenINFEngineA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszInfFilename", "pszInstallSection", "dwFlags", "phInf", "pvReserved"]),
        #
        'OpenINFEngineW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszInfFilename", "pszInstallSection", "dwFlags", "phInf", "pvReserved"]),
        #
        'TranslateInfStringExA': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hInf", "pszInfFilename", "pszTranslateSection", "pszTranslateKey", "pszBuffer", "dwBufferSize", "pdwRequiredSize", "pvReserved"]),
        #
        'TranslateInfStringExW': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hInf", "pszInfFilename", "pszTranslateSection", "pszTranslateKey", "pszBuffer", "dwBufferSize", "pdwRequiredSize", "pvReserved"]),
        #
        'CloseINFEngine': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hInf"]),
        #
        'ExtractFilesA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pszCabName", "pszExpandDir", "dwFlags", "pszFileList", "lpReserved", "dwReserved"]),
        #
        'ExtractFilesW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pszCabName", "pszExpandDir", "dwFlags", "pszFileList", "lpReserved", "dwReserved"]),
        #
        'LaunchINFSectionW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndOwner", "hInstance", "pszParams", "nShow"]),
        #
        'UserInstStubWrapperA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "hInstance", "pszParms", "nShow"]),
        #
        'UserInstStubWrapperW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "hInstance", "pszParms", "nShow"]),
        #
        'UserUnInstStubWrapperA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "hInstance", "pszParms", "nShow"]),
        #
        'UserUnInstStubWrapperW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "hInstance", "pszParms", "nShow"]),
        #
        'SetPerUserSecValuesA': SimTypeFunction([SimTypePointer(SimStruct({"szGUID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 59), "szDispName": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "szLocale": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 10), "szStub": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 1040), "szVersion": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 32), "szCompID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "dwIsInstalled": SimTypeInt(signed=False, label="UInt32"), "bRollback": SimTypeInt(signed=True, label="Int32")}, name="PERUSERSECTIONA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPerUser"]),
        #
        'SetPerUserSecValuesW': SimTypeFunction([SimTypePointer(SimStruct({"szGUID": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 59), "szDispName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "szLocale": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 10), "szStub": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 1040), "szVersion": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 32), "szCompID": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "dwIsInstalled": SimTypeInt(signed=False, label="UInt32"), "bRollback": SimTypeInt(signed=True, label="Int32")}, name="PERUSERSECTIONW", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPerUser"]),
    }

lib.set_prototypes(prototypes)



from . import SimConcretizationStrategy
from ..errors import SimMergeError


class SimConcretizationStrategyNorepeatsRange(SimConcretizationStrategy):
    """
    Concretization strategy that resolves a range, with no repeats.
    """

    def __init__(self, repeat_expr, min=None, granularity=None, **kwargs):  # pylint:disable=redefined-builtin
        super().__init__(**kwargs)
        self._repeat_expr = repeat_expr
        self._repeat_min = min
        self._repeat_granularity = granularity

    def _concretize(self, memory, addr, **kwargs):
        child_constraints = (addr >= self._repeat_min, addr < self._repeat_min + self._repeat_granularity)
        extra_constraints = kwargs.pop("extra_constraints", None)
        if extra_constraints is not None:
            child_constraints += tuple(extra_constraints)
        c = self._any(memory, addr, extra_constraints=child_constraints)
        self._repeat_min = c + self._repeat_granularity
        return [c]

    def copy(self):
        return SimConcretizationStrategyNorepeatsRange(
            repeat_expr=self._repeat_expr, min=self._repeat_min, granularity=self._repeat_granularity, exact=self._exact
        )

    def merge(self, others):
        if not all(self._repeat_expr is o._repeat_expr for o in others):
            raise SimMergeError("Unable to merge two different repeat expressions.")

        self._repeat_min = max(self._repeat_min, max(o._repeat_min for o in others))
        self._repeat_granularity = max(self._repeat_granularity, max(o._repeat_granularity for o in others))



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("mdmregistration.dll")
prototypes = \
    {
        #
        'GetDeviceRegistrationInfo': SimTypeFunction([SimTypeInt(signed=False, label="REGISTRATION_INFORMATION_CLASS"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceInformationClass", "ppDeviceRegistrationInfo"]),
        #
        'IsDeviceRegisteredWithManagement': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pfIsDeviceRegisteredWithManagement", "cchUPN", "pszUPN"]),
        #
        'IsManagementRegistrationAllowed': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pfIsManagementRegistrationAllowed"]),
        #
        'IsMdmUxWithoutAadAllowed': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["isEnrollmentAllowed"]),
        #
        'SetManagedExternally': SimTypeFunction([SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["IsManagedExternally"]),
        #
        'DiscoverManagementService': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pszMDMServiceUri": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszAuthenticationUri": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="MANAGEMENT_SERVICE_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszUPN", "ppMgmtInfo"]),
        #
        'RegisterDeviceWithManagementUsingAADCredentials': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserToken"]),
        #
        'RegisterDeviceWithManagementUsingAADDeviceCredentials': SimTypeFunction([], SimTypeInt(signed=True, label="Int32")),
        #
        'RegisterDeviceWithManagement': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszUPN", "ppszMDMServiceUri", "ppzsAccessToken"]),
        #
        'UnregisterDeviceWithManagement': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["enrollmentID"]),
        #
        'GetManagementAppHyperlink': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cchHyperlink", "pszHyperlink"]),
        #
        'DiscoverManagementServiceEx': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pszMDMServiceUri": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszAuthenticationUri": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="MANAGEMENT_SERVICE_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszUPN", "pszDiscoveryServiceCandidate", "ppMgmtInfo"]),
    }

lib.set_prototypes(prototypes)



# pylint: disable=missing-class-docstring
import claripy

import angr


class ExAllocatePool(angr.SimProcedure):
    def run(self, PoolType, NumberOfBytes):  # pylint:disable=arguments-differ, unused-argument
        addr = self.state.heap._malloc(NumberOfBytes)
        memset = angr.SIM_PROCEDURES["libc"]["memset"]
        self.inline_call(memset, addr, claripy.BVV(0, 8), NumberOfBytes)  # zerofill
        return addr



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("ninput.dll")
prototypes = \
    {
        #
        'CreateInteractionContext': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext"]),
        #
        'DestroyInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext"]),
        #
        'RegisterOutputCallbackInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"interactionId": SimTypeInt(signed=False, label="INTERACTION_ID"), "interactionFlags": SimTypeInt(signed=False, label="INTERACTION_FLAGS"), "inputType": SimTypeBottom(label="POINTER_INPUT_TYPE"), "x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32), "arguments": SimUnion({"manipulation": SimStruct({"delta": SimStruct({"translationX": SimTypeFloat(size=32), "translationY": SimTypeFloat(size=32), "scale": SimTypeFloat(size=32), "expansion": SimTypeFloat(size=32), "rotation": SimTypeFloat(size=32)}, name="MANIPULATION_TRANSFORM", pack=False, align=None), "cumulative": SimStruct({"translationX": SimTypeFloat(size=32), "translationY": SimTypeFloat(size=32), "scale": SimTypeFloat(size=32), "expansion": SimTypeFloat(size=32), "rotation": SimTypeFloat(size=32)}, name="MANIPULATION_TRANSFORM", pack=False, align=None), "velocity": SimStruct({"velocityX": SimTypeFloat(size=32), "velocityY": SimTypeFloat(size=32), "velocityExpansion": SimTypeFloat(size=32), "velocityAngular": SimTypeFloat(size=32)}, name="MANIPULATION_VELOCITY", pack=False, align=None), "railsState": SimTypeInt(signed=False, label="MANIPULATION_RAILS_STATE")}, name="INTERACTION_ARGUMENTS_MANIPULATION", pack=False, align=None), "tap": SimStruct({"count": SimTypeInt(signed=False, label="UInt32")}, name="INTERACTION_ARGUMENTS_TAP", pack=False, align=None), "crossSlide": SimStruct({"flags": SimTypeInt(signed=False, label="CROSS_SLIDE_FLAGS")}, name="INTERACTION_ARGUMENTS_CROSS_SLIDE", pack=False, align=None)}, name="<anon>", label="None")}, name="INTERACTION_CONTEXT_OUTPUT", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["clientData", "output"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "outputCallback", "clientData"]),
        #
        'RegisterOutputCallbackInteractionContext2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"interactionId": SimTypeInt(signed=False, label="INTERACTION_ID"), "interactionFlags": SimTypeInt(signed=False, label="INTERACTION_FLAGS"), "inputType": SimTypeBottom(label="POINTER_INPUT_TYPE"), "contactCount": SimTypeInt(signed=False, label="UInt32"), "currentContactCount": SimTypeInt(signed=False, label="UInt32"), "x": SimTypeFloat(size=32), "y": SimTypeFloat(size=32), "arguments": SimUnion({"manipulation": SimStruct({"delta": SimStruct({"translationX": SimTypeFloat(size=32), "translationY": SimTypeFloat(size=32), "scale": SimTypeFloat(size=32), "expansion": SimTypeFloat(size=32), "rotation": SimTypeFloat(size=32)}, name="MANIPULATION_TRANSFORM", pack=False, align=None), "cumulative": SimStruct({"translationX": SimTypeFloat(size=32), "translationY": SimTypeFloat(size=32), "scale": SimTypeFloat(size=32), "expansion": SimTypeFloat(size=32), "rotation": SimTypeFloat(size=32)}, name="MANIPULATION_TRANSFORM", pack=False, align=None), "velocity": SimStruct({"velocityX": SimTypeFloat(size=32), "velocityY": SimTypeFloat(size=32), "velocityExpansion": SimTypeFloat(size=32), "velocityAngular": SimTypeFloat(size=32)}, name="MANIPULATION_VELOCITY", pack=False, align=None), "railsState": SimTypeInt(signed=False, label="MANIPULATION_RAILS_STATE")}, name="INTERACTION_ARGUMENTS_MANIPULATION", pack=False, align=None), "tap": SimStruct({"count": SimTypeInt(signed=False, label="UInt32")}, name="INTERACTION_ARGUMENTS_TAP", pack=False, align=None), "crossSlide": SimStruct({"flags": SimTypeInt(signed=False, label="CROSS_SLIDE_FLAGS")}, name="INTERACTION_ARGUMENTS_CROSS_SLIDE", pack=False, align=None)}, name="<anon>", label="None")}, name="INTERACTION_CONTEXT_OUTPUT2", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["clientData", "output"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "outputCallback", "clientData"]),
        #
        'SetInteractionConfigurationInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"interactionId": SimTypeInt(signed=False, label="INTERACTION_ID"), "enable": SimTypeInt(signed=False, label="INTERACTION_CONFIGURATION_FLAGS")}, name="INTERACTION_CONTEXT_CONFIGURATION", pack=False, align=None), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "configurationCount", "configuration"]),
        #
        'GetInteractionConfigurationInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"interactionId": SimTypeInt(signed=False, label="INTERACTION_ID"), "enable": SimTypeInt(signed=False, label="INTERACTION_CONFIGURATION_FLAGS")}, name="INTERACTION_CONTEXT_CONFIGURATION", pack=False, align=None), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "configurationCount", "configuration"]),
        #
        'SetPropertyInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="INTERACTION_CONTEXT_PROPERTY"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "contextProperty", "value"]),
        #
        'GetPropertyInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="INTERACTION_CONTEXT_PROPERTY"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "contextProperty", "value"]),
        #
        'SetInertiaParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="INERTIA_PARAMETER"), SimTypeFloat(size=32)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "inertiaParameter", "value"]),
        #
        'GetInertiaParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="INERTIA_PARAMETER"), SimTypePointer(SimTypeFloat(size=32), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "inertiaParameter", "value"]),
        #
        'SetCrossSlideParametersInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"threshold": SimTypeInt(signed=False, label="CROSS_SLIDE_THRESHOLD"), "distance": SimTypeFloat(size=32)}, name="CROSS_SLIDE_PARAMETER", pack=False, align=None), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameterCount", "crossSlideParameters"]),
        #
        'GetCrossSlideParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="CROSS_SLIDE_THRESHOLD"), SimTypePointer(SimTypeFloat(size=32), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "threshold", "distance"]),
        #
        'SetTapParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="TAP_PARAMETER"), SimTypeFloat(size=32)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameter", "value"]),
        #
        'GetTapParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="TAP_PARAMETER"), SimTypePointer(SimTypeFloat(size=32), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameter", "value"]),
        #
        'SetHoldParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="HOLD_PARAMETER"), SimTypeFloat(size=32)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameter", "value"]),
        #
        'GetHoldParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="HOLD_PARAMETER"), SimTypePointer(SimTypeFloat(size=32), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameter", "value"]),
        #
        'SetTranslationParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="TRANSLATION_PARAMETER"), SimTypeFloat(size=32)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameter", "value"]),
        #
        'GetTranslationParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="TRANSLATION_PARAMETER"), SimTypePointer(SimTypeFloat(size=32), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameter", "value"]),
        #
        'SetMouseWheelParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="MOUSE_WHEEL_PARAMETER"), SimTypeFloat(size=32)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameter", "value"]),
        #
        'GetMouseWheelParameterInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="MOUSE_WHEEL_PARAMETER"), SimTypePointer(SimTypeFloat(size=32), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "parameter", "value"]),
        #
        'ResetInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext"]),
        #
        'GetStateInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"pointerType": SimTypeBottom(label="POINTER_INPUT_TYPE"), "pointerId": SimTypeInt(signed=False, label="UInt32"), "frameId": SimTypeInt(signed=False, label="UInt32"), "pointerFlags": SimTypeInt(signed=False, label="POINTER_FLAGS"), "sourceDevice": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hwndTarget": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "ptPixelLocation": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptHimetricLocation": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptPixelLocationRaw": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptHimetricLocationRaw": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "dwTime": SimTypeInt(signed=False, label="UInt32"), "historyCount": SimTypeInt(signed=False, label="UInt32"), "InputData": SimTypeInt(signed=True, label="Int32"), "dwKeyStates": SimTypeInt(signed=False, label="UInt32"), "PerformanceCount": SimTypeLongLong(signed=False, label="UInt64"), "ButtonChangeType": SimTypeInt(signed=False, label="POINTER_BUTTON_CHANGE_TYPE")}, name="POINTER_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="INTERACTION_STATE"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "pointerInfo", "state"]),
        #
        'AddPointerInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "pointerId"]),
        #
        'RemovePointerInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "pointerId"]),
        #
        'ProcessPointerFramesInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"pointerType": SimTypeBottom(label="POINTER_INPUT_TYPE"), "pointerId": SimTypeInt(signed=False, label="UInt32"), "frameId": SimTypeInt(signed=False, label="UInt32"), "pointerFlags": SimTypeInt(signed=False, label="POINTER_FLAGS"), "sourceDevice": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hwndTarget": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "ptPixelLocation": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptHimetricLocation": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptPixelLocationRaw": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptHimetricLocationRaw": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "dwTime": SimTypeInt(signed=False, label="UInt32"), "historyCount": SimTypeInt(signed=False, label="UInt32"), "InputData": SimTypeInt(signed=True, label="Int32"), "dwKeyStates": SimTypeInt(signed=False, label="UInt32"), "PerformanceCount": SimTypeLongLong(signed=False, label="UInt64"), "ButtonChangeType": SimTypeInt(signed=False, label="POINTER_BUTTON_CHANGE_TYPE")}, name="POINTER_INFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "entriesCount", "pointerCount", "pointerInfo"]),
        #
        'BufferPointerPacketsInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"pointerType": SimTypeBottom(label="POINTER_INPUT_TYPE"), "pointerId": SimTypeInt(signed=False, label="UInt32"), "frameId": SimTypeInt(signed=False, label="UInt32"), "pointerFlags": SimTypeInt(signed=False, label="POINTER_FLAGS"), "sourceDevice": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hwndTarget": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "ptPixelLocation": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptHimetricLocation": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptPixelLocationRaw": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "ptHimetricLocationRaw": SimStruct({"x": SimTypeInt(signed=True, label="Int32"), "y": SimTypeInt(signed=True, label="Int32")}, name="POINT", pack=False, align=None), "dwTime": SimTypeInt(signed=False, label="UInt32"), "historyCount": SimTypeInt(signed=False, label="UInt32"), "InputData": SimTypeInt(signed=True, label="Int32"), "dwKeyStates": SimTypeInt(signed=False, label="UInt32"), "PerformanceCount": SimTypeLongLong(signed=False, label="UInt64"), "ButtonChangeType": SimTypeInt(signed=False, label="POINTER_BUTTON_CHANGE_TYPE")}, name="POINTER_INFO", pack=False, align=None), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "entriesCount", "pointerInfo"]),
        #
        'ProcessBufferedPacketsInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext"]),
        #
        'ProcessInertiaInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext"]),
        #
        'StopInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext"]),
        #
        'SetPivotInteractionContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeFloat(size=32), SimTypeFloat(size=32), SimTypeFloat(size=32)], SimTypeInt(signed=True, label="Int32"), arg_names=["interactionContext", "x", "y", "radius"]),
    }

lib.set_prototypes(prototypes)
